#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Foot Print
#      By:	sdmitch
#   Usage:	Create an outline of buildings and other surface features
#	 Note:	Needs some way to remove duplicate edges caused by top and bottom edges of features.
#			
#    Date:	Jul 2011
#------------------------------------------------------------------------------------------------
#
require 'Sketchup'
#
filename=File.basename(__FILE__)
#
if not file_loaded?(filename)
    UI.menu('Plugins').add_item('Foot Print') { SDM_FP.footprint }
end
#
file_loaded(filename)
#
module SDM_FP

	def SDM_FP.footprint
		@Mod=Sketchup.active_model; @Ent=@Mod.entities; @Sel=@Mod.selection
		@Mod.start_operation "Foot Print"
		@edges=[]; @bottom=1e6; 
		SDM_FP.get_edges(@Sel); 
		@bottom=@bottom.floor - 1; @Sel.clear
		plane = [Geom::Point3d.new(0, 0 ,@bottom), Geom::Vector3d.new(0,0,1)]
		for e in @edges
			p1=e.start.position.project_to_plane plane
			p2=e.end.position.project_to_plane plane
			edge=@Ent.add_line(p1,p2)
			@Sel.add edge if edge
		end
		save=@Sel.to_a; @Sel.clear
		save.each{|e| e.find_faces}
		save.each{|e| @Sel.add(e) if e.faces.length>1}
		@Ent.erase_entities  @Sel
		@Mod.commit_operation
	end
		
	def SDM_FP.get_edges(ents)
		ents.each do |e|
			if e.is_a? Sketchup::Group
				tr=Geom::Transformation.new(e.bounds.min)
				e.entities.each do |g|
					if g.is_a? Sketchup::Edge
						pt,vec=g.line
						if vec.z.abs < 1.0 then
							p1=g.start.position.transform tr
							p2=g.end.position.transform tr
							edge =  @Ent.add_line(p1,p2)
							 if edge
								@edges << edge if !@edges.include? edge
								minz = [edge.start.position.z,edge.end.position.z].min
								@bottom = minz if @bottom > minz
							end
						end
					end
				end
			elsif e.is_a? Sketchup::ComponentInstance
				tr=Geom::Transformation.new(e.bounds.min)
				e.definition.entities.each do |c|
					if c.is_a? Sketchup::Edge
						pt,vec=c.line
						if vec.z.abs < 1.0 then
							p1=c.start.position.transform tr
							p2=c.end.position.transform tr
							edge =  @Ent.add_line(p1,p2)
							 if edge
								@edges << edge if !@edges.include? edge
								minz = [edge.start.position.z,edge.end.position.z].min
								@bottom = minz if @bottom > minz
							end
						end
					end
				end
			elsif e.is_a? Sketchup::Edge
				pt,vec=e.line
				if vec.z.abs < 1.0 then
					@edges << e if !@edges.include? e
					minz = [e.start.position.z,e.end.position.z].min
					@bottom = minz if @bottom > minz
				end
			end
		end
	end
	
end

